#include <Application.h>
#include <Entry.h>
#include <Path.h>
#include <string.h>
#include <stdio.h>

#include "MyWindow.h"
#include "convert.h"
#include "lang.h"

int convert(char in_type, char in_end_line_type, const char *in_file_path, char out_type, char out_end_line_type, const char *out_file_path) {
	if (in_end_line_type == 0 && out_end_line_type != 0) {
		(new BAlert("", ALERT_END_LINE_ERR, "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_OFFSET_SPACING, B_WARNING_ALERT))->Go();
		out_end_line_type = 0;
	}
	
	FILE *in_file, *out_file;
	
	if ( !strcasecmp(in_file_path, out_file_path) ) {
		(new BAlert("", ALERT_FILE_ERR, "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_OFFSET_SPACING, B_STOP_ALERT))->Go();
		return 1;
	}
	
	in_file = fopen(in_file_path, "r");
	if (in_file == NULL) {
		(new BAlert("", ALERT_IN_FILE_ERR, "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_OFFSET_SPACING, B_STOP_ALERT))->Go();
		return 1;
	}
	
	out_file = fopen(out_file_path, "wb");
	if (out_file == NULL) {
		(new BAlert("", ALERT_OUT_FILE_ERR, "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_OFFSET_SPACING, B_STOP_ALERT))->Go();
		return 1;
	}
	
	int control = convert_in(in_type, in_end_line_type, in_file, out_type, out_end_line_type, out_file);

	if (control == 0) {
		(new BAlert("", ALERT_CONV_SUCSES, "OK"))->Go();
	} else if (control == 2) {
		(new BAlert("", ALERT_UTF16_NO_DETECT_ERR, "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_OFFSET_SPACING, B_STOP_ALERT))->Go();
		return 1;
	}
	
	fclose(in_file);
	fclose(out_file);
}


int in_type=1, in_end_line_type=0, out_type=1, out_end_line_type=0;

/***** Constructor ****/
MyWindow::MyWindow(BRect frame,const char *title,window_look look,window_feel feel,uint32 flags,uint32 workspace)
: BWindow(frame,title,look,feel,flags,workspace)
{
	
	// Input encoding
	select_menu_in_type = new BMenu("");
	select_menu_in_type->AddItem(new BMenuItem("UTF-7", new BMessage(IN_1)));
	select_menu_in_type->AddItem(new BMenuItem("UTF-8", new BMessage(IN_1)));
	select_menu_in_type->AddItem(new BMenuItem("UTF-16 autodetect", new BMessage(IN_1)));
	select_menu_in_type->AddItem(new BMenuItem("UTF-16 be", new BMessage(IN_1)));
	select_menu_in_type->AddItem(new BMenuItem("UTF-16 le", new BMessage(IN_1)));
	
	select_menu_in_type->SetLabelFromMarked(true);
	(select_menu_in_type->ItemAt(1))->SetMarked(true);
	select_in_type = new BMenuField(BRect(15, 15, 220, 35), "in_type", IN_TYPE, select_menu_in_type);
	AddChild(select_in_type);
	
	// Input endline
	select_menu_in_endline = new BMenu("");
	select_menu_in_endline->AddItem(new BMenuItem("-", new BMessage(IN_2)));
	select_menu_in_endline->AddItem(new BMenuItem("LF", new BMessage(IN_2)));
	select_menu_in_endline->AddItem(new BMenuItem("CR", new BMessage(IN_2)));
	select_menu_in_endline->AddItem(new BMenuItem("CRLF", new BMessage(IN_2)));
	
	select_menu_in_endline->SetLabelFromMarked(true);
	(select_menu_in_endline->ItemAt(0))->SetMarked(true);
	select_in_endline = new BMenuField(BRect(15, 40, 220, 60), "in_endline", IN_END_LINE, select_menu_in_endline);
	AddChild(select_in_endline);

	// Output encoding
	select_menu_out_type = new BMenu("");
	select_menu_out_type->AddItem(new BMenuItem("UTF-7", new BMessage(OUT_1)));
	select_menu_out_type->AddItem(new BMenuItem("UTF-8", new BMessage(OUT_1)));
	select_menu_out_type->AddItem(new BMenuItem("UTF-16 be", new BMessage(OUT_1)));
	select_menu_out_type->AddItem(new BMenuItem("UTF-16 le", new BMessage(OUT_1)));
	
	select_menu_out_type->SetLabelFromMarked(true);
	(select_menu_out_type->ItemAt(1))->SetMarked(true);
	select_out_type = new BMenuField(BRect(240, 15, 445, 35), "out_type", OUT_TYPE, select_menu_out_type);
	AddChild(select_out_type);
	
	// Output endline
	select_menu_out_endline = new BMenu("");
	select_menu_out_endline->AddItem(new BMenuItem("-", new BMessage(OUT_2)));
	select_menu_out_endline->AddItem(new BMenuItem("LF", new BMessage(OUT_2)));
	select_menu_out_endline->AddItem(new BMenuItem("CR", new BMessage(OUT_2)));
	select_menu_out_endline->AddItem(new BMenuItem("CRLF", new BMessage(OUT_2)));
	
	select_menu_out_endline->SetLabelFromMarked(true);
	(select_menu_out_endline->ItemAt(0))->SetMarked(true);
	select_out_endline = new BMenuField(BRect(240, 40, 445, 60), "out_endline", OUT_END_LINE, select_menu_out_endline);
	AddChild(select_out_endline);

	// infile path
	in_path_text = new BTextControl(BRect(15, 70, 220, 90), "in_path", IN_PATH, NULL, new BMessage(IN_PATH_TEXT));
	AddChild(in_path_text);
	
	// outfile path
	out_path_text = new BTextControl(BRect(240, 70, 445, 90), "out_path", OUT_PATH, NULL, new BMessage(OUT_PATH_TEXT));
	AddChild(out_path_text);

	// convert button
	AddChild( new BButton( BRect(188, 105, 273, 135), "convert", CONVERT, new BMessage( MSG_PRZYSISK_1 ) ) );
}

// messaes system
void MyWindow::MessageReceived(BMessage *message)
{
	// drag and drop (1)
	entry_ref ref;
	
	switch (message->what){
	// drag and drop (2)
	case B_SIMPLE_DATA:
		if( message->FindRef("refs", &ref) == B_OK ){
			BEntry entry(&ref);
			BPath path(&entry);
			char p[500] = "";
			strcat(p, path.Path());
			in_path_text->SetText( p );
			strcat(p, ".cnv");
			out_path_text->SetText( p );
		}
	break;
	case MSG_PRZYSISK_1:
		convert(in_type, in_end_line_type, in_path_text->Text(), out_type, out_end_line_type, out_path_text->Text());
	break;
	case IN_1:
		message->FindInt32((const char *)"index", (int32 *)&in_type);
	break;
	case IN_2:
		message->FindInt32((const char *)"index", (int32 *)&in_end_line_type);
	break;
	case OUT_1:
		message->FindInt32((const char *)"index", (int32 *)&out_type);
		if (out_type > 1) out_type++;
	break;
	case OUT_2:
		message->FindInt32((const char *)"index", (int32 *)&out_end_line_type);
	break;
	default:
		BWindow::MessageReceived(message);
	break;
	}
}

/***** QuitRequested ****/
bool MyWindow::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return(true);
}

/***** start GUI ****/
utf_convert_gui::utf_convert_gui(const char *signature)	: BApplication(signature)
{
	_mywindow = new MyWindow(BRect(230,148,690,294), WINDOW_TITLE, B_TITLED_WINDOW_LOOK, B_NORMAL_WINDOW_FEEL, B_NOT_ZOOMABLE | B_NOT_RESIZABLE, 0);
	_mywindow->Show();
}