/*
	Copyright (C) 2003 Robert Paciorek ( http://robert.n17.waw.pl )
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
	USA.
	
	http://www.gnu.org
*/


/*
	program based (functions: utf7_getc, utf7_putc, utf8_getc, utf8_putc and invert) on programs by
	* Ross Paterson <rap@doc.ic.ac.uk>, Department of Computing, Imperial College, London SW7
	* and Guo Jin <guojin@iss.nus.sg> modification
	
	( thats programs not include any licence informations, it seems most likely in the public domain )
	
	orginal code from ftp://unicode.org/pub/Programs/ConvertUTF/
		or: http://www.ibiblio.org/pub/packages/ccic/software/unix/convert/utf7/
*/

/*
 * The UTF-FSS (aka UTF-2) encoding of UCS, as described in the following
 * quote from Ken Thompson's utf-fss.c:
 *
 * Bits  Hex Min  Hex Max  Byte Sequence in Binary
 *   7  00000000 0000007f 0vvvvvvv
 *  11  00000080 000007FF 110vvvvv 10vvvvvv
 *  16  00000800 0000FFFF 1110vvvv 10vvvvvv 10vvvvvv
 *  21  00010000 001FFFFF 11110vvv 10vvvvvv 10vvvvvv 10vvvvvv
 *  26  00200000 03FFFFFF 111110vv 10vvvvvv 10vvvvvv 10vvvvvv 10vvvvvv
 *  31  04000000 7FFFFFFF 1111110v 10vvvvvv 10vvvvvv 10vvvvvv 10vvvvvv 10vvvvvv
 *
 * The UCS value is just the concatenation of the v bits in the multibyte
 * encoding.  When there are multiple ways to encode a value, for example
 * UCS 0, only the shortest encoding is legal.
 */


#include "convert.h"


/****       *  CONVERT  *       ****/

// out_type encoding
int convert_out(Char in_char, char out_type, FILE *out_file)
/*
	function return:
	* 0 if sucsessfull
	* 1 if in-out type error
*/
{
	switch (out_type) {
		case UTF7:
			utf7_putc(in_char, out_file);
		break;
		case UTF8:
			utf8_putc(in_char, out_file);
		break;
		case UTF16be:
			utf16be_putc(in_char, out_file);
		break;
		case UTF16le:
			utf16le_putc(in_char, out_file);
		break;
		default:
			return 1;
		break;
	}
	return 0;
}


// in_type encoding and read file (while ...)
int convert_in(char in_type, char in_end_line_type, FILE *in_file, char out_type, char out_end_line_type, FILE *out_file)
/*
	function return:
	* 0 if sucsessfull
	* 1 if in-out type error
	* 2 if UTF16_NO_DETECT_ERR
	OR error of end_line function
*/
{
	unsigned short	unichar;
	unsigned short	unichar_1;
	unsigned short	unichar_2;
	int control;

	switch (in_type) {
		case UTF7:
			while ((unichar = utf7_getc(in_file)) != (unsigned short)EOF) {
				control = convert_end_line(unichar, in_end_line_type, out_type, out_end_line_type, out_file);
				if ( control !=0 ) return control;
			}
		break;
		case UTF8:
			while ((unichar = utf8_getc(in_file)) != (unsigned short)EOF) {
				control = convert_end_line(unichar, in_end_line_type, out_type, out_end_line_type, out_file);
				if ( control !=0 ) return control;
			}
		break;
		case UTF16:
			unichar_1 = getc(in_file);
			unichar_2 = getc(in_file);
			// from UTF-16be
			if (unichar_1==0xfe && unichar_2==0xff){
				while ((unichar = utf16be_getc(in_file)) != (unsigned short)EOF) {
					control = convert_end_line(unichar, in_end_line_type, out_type, out_end_line_type, out_file);
					if ( control !=0 ) return control;
				}
			// from UTF-16le
			} else if (unichar_1==0xff && unichar_2==0xfe){
				while ((unichar = utf16le_getc(in_file)) != (unsigned short)EOF) {
					control = convert_end_line(unichar, in_end_line_type, out_type, out_end_line_type, out_file);
					if ( control !=0 ) return control;
				}
			// no detect
			} else {
				return 2;
			}
		break;
		case UTF16be:
			while ((unichar = utf16be_getc(in_file)) != (unsigned short)EOF) {
				control = convert_end_line(unichar, in_end_line_type, out_type, out_end_line_type, out_file);
				if ( control !=0 ) return control;
			}
		break;
		case UTF16le:
			while ((unichar = utf16le_getc(in_file)) != (unsigned short)EOF) {
				control = convert_end_line(unichar, in_end_line_type, out_type, out_end_line_type, out_file);
				if ( control !=0 ) return control;
			}
		default:
			// in-out type error
			return 1;
		break;
	}
	
	// xxxxxxxxxxxxxxxx
	if ( out_type == UTF7 ) {
		utf7_putc(0x20, out_file);
	}
	
	// convert sucsesfull
	return 0;
}
