/*
	Copyright (C) 2003 Robert Paciorek ( http://robert.n17.waw.pl )
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
	USA.
	
	http://www.gnu.org
*/


/*
	program based (functions: utf7_getc, utf7_putc, utf8_getc, utf8_putc and invert) on programs by
	* Ross Paterson <rap@doc.ic.ac.uk>, Department of Computing, Imperial College, London SW7
	* and Guo Jin <guojin@iss.nus.sg> modification
	
	( thats programs not include any licence informations, it seems most likely in the public domain )
	
	orginal code from ftp://unicode.org/pub/Programs/ConvertUTF/
		or: http://www.ibiblio.org/pub/packages/ccic/software/unix/convert/utf7/
*/

/*
 * The UTF-FSS (aka UTF-2) encoding of UCS, as described in the following
 * quote from Ken Thompson's utf-fss.c:
 *
 * Bits  Hex Min  Hex Max  Byte Sequence in Binary
 *   7  00000000 0000007f 0vvvvvvv
 *  11  00000080 000007FF 110vvvvv 10vvvvvv
 *  16  00000800 0000FFFF 1110vvvv 10vvvvvv 10vvvvvv
 *  21  00010000 001FFFFF 11110vvv 10vvvvvv 10vvvvvv 10vvvvvv
 *  26  00200000 03FFFFFF 111110vv 10vvvvvv 10vvvvvv 10vvvvvv 10vvvvvv
 *  31  04000000 7FFFFFFF 1111110v 10vvvvvv 10vvvvvv 10vvvvvv 10vvvvvv 10vvvvvv
 *
 * The UCS value is just the concatenation of the v bits in the multibyte
 * encoding.  When there are multiple ways to encode a value, for example
 * UCS 0, only the shortest encoding is legal.
 */

#include <Application.h>
#include "MyWindow.h"
#include "convert.h"
#include "lang.h"


int main(int argc, char *argv[])
{
	// start GUI
	if ( argv[1]==NULL ) {
		utf_convert_gui application("application/utf_convert_gui");
		application.Run();
	// print help
	} else if ( argv[2]==NULL || argv[3]==NULL || argv[4]==NULL ) {
		printf( HELP );
		exit(0);
	// start text mode
	} else {
		FILE	*ifp, *ofp;
		int	control;
		char in_type=1, in_end_line_type=0, out_type=1, out_end_line_type=0;

		// convert text arg to num args for convert_in function
		     if ( !strcasecmp(argv[1],"utf7") )	in_type = UTF7;
		else if ( !strcasecmp(argv[1],"utf8") )	in_type = UTF8;
		else if ( !strcasecmp(argv[1],"utf16") )	in_type = UTF16;
		else if ( !strcasecmp(argv[1],"utf16be") )	in_type = UTF16be;
		else if ( !strcasecmp(argv[1],"utf16le") )	in_type = UTF16le;
		
		     if ( !strcasecmp(argv[2],"-") )	in_end_line_type = NOT;
		else if ( !strcasecmp(argv[2],"lf") )	in_end_line_type = LF;
		else if ( !strcasecmp(argv[2],"cr") )	in_end_line_type = CR;
		else if ( !strcasecmp(argv[2],"crlf") )	in_end_line_type = CRLF;
		
		     if ( !strcasecmp(argv[3],"utf7") )	out_type = UTF7;
		else if ( !strcasecmp(argv[3],"utf8") )	out_type = UTF8;
		else if ( !strcasecmp(argv[3],"utf16be") )	out_type = UTF16be;
		else if ( !strcasecmp(argv[3],"utf16le") )	out_type = UTF16le;
		
		     if ( !strcasecmp(argv[4],"-") )	out_end_line_type = NOT;
		else if ( !strcasecmp(argv[4],"lf") )	out_end_line_type = LF;
		else if ( !strcasecmp(argv[4],"cr") )	out_end_line_type = CR;
		else if ( !strcasecmp(argv[4],"crlf") )	out_end_line_type = CRLF;

		// conrtol infile != outfile
		if ( argc > 6 && !strcasecmp(argv[5],argv[6]) ) {
			printf( FILE_ERR );
			exit(0);
		}
	
		// input file
		if (argv[5] != NULL && argc > 5){
			ifp = fopen(argv[5], "r");
			if (ifp == NULL) {
				printf( IN_FILE_ERR , argv[5]);
				exit(0);
			}
		} else {
			ifp = stdin;
		}
		
		// output file
		if (argv[6] != NULL && argc > 6){
			ofp = fopen(argv[6], "wb");
			if (ofp == NULL) {
				printf( OUT_FILE_ERR, argv[6]);
				exit(0);
			}
		} else {
			ofp = stdout;
		}

		// convert file
		control = convert_in(in_type, in_end_line_type, ifp, out_type, out_end_line_type, ofp);
		
		// return code control
		if (control == 1) {
			printf( HELP );
			exit(0);
		} else if (control == 2) {
			printf( UTF16_NO_DETECT_ERR );
			exit(0);
		} else if (control == 3) {
			printf( END_LINE_ERR );
			exit(0);
		}
		
		// end program ;-)
		fclose(ifp);
		fclose(ofp);
	}
	return 0;
}