/****       *  UTF-7  *       ****/

#include "convert.h"

// character classes
#define	BASE64	0x01
#define	SAFE	0x02
#define	OPTIONAL 0x04
#define	SPACE	0x08

#define	NUMBYTES 256
#define	CHARSIZE 16

// ASCII subsets
static	const	unsigned char	base64[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
static	const	unsigned char	safe[] = "'(),-.:?";
static	const	unsigned char	optional[] = "!\"#$%&*;<=>@[]^_`{|}";
static	const	unsigned char	space[] = " \t\n\r";

static	char	inv_base64[128];
static	char	char_type[NUMBYTES];

static	unsigned long	bit_buffer_get;
static	unsigned long	bit_buffer_put;
static	int	nbits_get;
static	int	nbits_put;

static	bool	in_base64_get;
static	bool	in_base64_put;
static	bool	initialized_get;
static	bool	initialized_put;

static void invert()
{
	int	i;
	const	unsigned char	*s;
	
	for (i = 0; i < NUMBYTES; i++)
		char_type[i] = 0;
	for (s = base64; *s != '\0'; s++) {
		char_type[*s] |= BASE64;
		inv_base64[*s] = s - base64;
	}
	for (s = safe; *s != '\0'; s++)
		char_type[*s] |= SAFE;
	for (s = optional; *s != '\0'; s++)
		char_type[*s] |= OPTIONAL;
	for (s = space; *s != '\0'; s++)
		char_type[*s] |= SPACE;
}

// get unicode from UTF-7
Char utf7_getc(FILE *f)
{
	int	c;

	if (! initialized_get) {
		invert();
		initialized_get = true;
	}
	for (;;) {
		if ((c = getc(f)) == EOF)
			return EOF;
		if (! in_base64_get) {
			if (c != '+')
				return c;
			if ((c = getc(f)) == EOF)
				return EOF;
			if (c == '-')
				return '+';
			in_base64_get = true;
			nbits_get = 0;
		}
		/* now we're in Base64 mode */
		while (char_type[c]&BASE64) {
			bit_buffer_get <<= 6;
			bit_buffer_get |= inv_base64[c];
			nbits_get += 6;
			if (nbits_get >= 16) {
				nbits_get -= 16;
				return ((bit_buffer_get >> nbits_get)&0xffffL);
			}
			if ((c = getc(f)) == EOF)
				return EOF;
		}
		in_base64_get = false;
		if (c != '-')
			return c;
	}
}

// put unicode to UTF-7
int utf7_putc(Char c, FILE *f)
{
	if (! initialized_get) {
		invert();
		initialized_put = true;
	}
	if (c <= 0x7f && (char_type[c] & (BASE64|SAFE|SPACE))) {
		if (in_base64_put) {
			if (nbits_put > 0)
				putc(base64[(bit_buffer_put<<(6-nbits_put))&0x3f], f);
			if ((char_type[c] & BASE64) || c == '-')
				putchar('-');
			in_base64_put = false;
		}
		putc(c, f);
		if (c == '+')
			putc('-', f);
	}
	else {
		if (! in_base64_put) {
			putc('+', f);
			in_base64_put = true;
			nbits_put = 0;
		}
		bit_buffer_put <<= CHARSIZE;
		bit_buffer_put |= c;
		nbits_put += CHARSIZE;
		while (nbits_put >= 6) {
			nbits_put -= 6;
			putc(base64[(bit_buffer_put >> nbits_put)&0x3f], f);
		}
	}
	return 0;
}