/****       *  UTF-8  *       ****/

#include "convert.h"

// get unicode from UTF-8
Char utf8_getc(FILE *f)
{
	#define	CODE_ERROR	0x80
	
	register	int	c;
	register	Char	wc;
	register	int	extras;
	register	unsigned	bit;

	// get char
	c = getc(f);
	
	// EOF - end file
	if (c == EOF)
		return EOF;
	// ASCII character
	if ((c & 0x80) == 0)
		return c;
	// ERROR - unexpected tail character 
	if ((c & 0xc0) == 0x80)
		return CODE_ERROR;

	// how many extra bytes?
	extras = 1;
	for (bit = 0x20; (c & bit) != 0; bit >>= 1)
		extras++;
	if (extras > 5)
		return CODE_ERROR;
	
	// put header byte
	wc = c & (bit-1);
	
	// put tail bytes
	while (extras-- > 0) {
		if ((c = getc(f)) == EOF)
			return EOF;
		if ((c & 0xc0) != 0x80) { /* unexpected head character */
			ungetc(c, f);
			return CODE_ERROR;
		}
		wc = (wc<<6) | c&0x3f;
	}
	
	return wc;
}

// put unicode to UTF-8
int utf8_putc(Char wc, FILE *f)
{
	register	Char	tmp;
	register	int	extras;

	wc &= 0x7fffffffL;
	
	// ASCII character
	if ((wc & 0x7f) == wc)
		return putc((int)wc, f) == EOF ? EOF : wc;
	
	// how many extra bytes are required?
	extras = 1;
	for (tmp = wc >> 11; tmp != 0; tmp >>= 5)
		extras++;
	
	// put header byte
	if (putc(0xff&(0x1f80 >> extras) | (int)(wc >> (extras*6)), f) == EOF)
		return EOF;

	// put tail bytes
	while (extras-- > 0)
		if (putc(0x80|0x3f&(int)(wc >> (extras*6)), f) == EOF)
			return EOF;
	
	return 0;
}